/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.aaa;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.dream.aaa.agent.PushNotification;
import org.objectweb.dream.aaa.components.conf.AgentServerConfiguration;
import org.objectweb.dream.aaa.components.conf.AgentServerInitialization;
import org.objectweb.dream.adl.FactoryFactory;
import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.util.Fractal;

/**
 * Start an agent server
 */
public final class AgentServer
{

  private AgentServer()
  {
  }

  /**
   * Create and start a local agent server
   * 
   * @param a3serverURL the URL pointing on the a3server configuration file.
   * @param serverName the name of the local server to start.
   * @return the component interface of the created server.
   * @throws Exception if an error occurs.
   */
  public static Component createAgentServer(String a3serverURL,
      String serverName) throws Exception
  {
    Factory f = FactoryFactory.getFactory();
    Map ctx = new HashMap();
    Component server = (Component) f.newComponent(
        "org.objectweb.dream.aaa.AgentServer", ctx);
    Fractal.getNameController(server).setFcName(serverName);
    AgentServerInitialization agentServerInitialization = (AgentServerInitialization) server
        .getFcInterface("initialize");
    Fractal.getLifeCycleController(server).startFc();

    agentServerInitialization.initialize(a3serverURL, serverName);

    return server;
  }

  /**
   * Returns the {@link PushNotification PushNotification}of the given server
   * component.
   * 
   * @param server a server component.
   * @return the {@link PushNotification PushNotification}of the given server
   *         component.
   */
  public static PushNotification getPushNotificationItf(Component server)
  {
    try
    {
      return (PushNotification) server.getFcInterface("inPushNotification");
    }
    catch (NoSuchInterfaceException e)
    {
      throw new IllegalArgumentException(
          "the given server component has no \"inPushNotification\" interface");
    }
  }

  /**
   * Returns the identifier of the default repository of the given server.
   * 
   * @param server a server component.
   * @return the identifier of the default repository of the given server.
   */
  public static short getDefaultRepositoryId(Component server)
  {
    AgentServerConfiguration agentServerConfigurationItf = getAgentServerConfigurationItf(server);
    return agentServerConfigurationItf.getDefaultRepositoryDesc()
        .getRepositoryID();
  }

  /**
   * Returns the identifier of the repository with the specified name, of the
   * given server.
   * 
   * @param server a server component.
   * @param repositoryName the name of the repository.
   * @return the identifier of the repository with the specified name, of the
   *         given server.
   */
  public static short getRepositoryId(Component server, String repositoryName)
  {
    AgentServerConfiguration agentServerConfigurationItf = getAgentServerConfigurationItf(server);
    return agentServerConfigurationItf.getRepositoryDesc(repositoryName)
        .getRepositoryID();
  }

  private static AgentServerConfiguration getAgentServerConfigurationItf(
      Component server)
  {
    try
    {
      return (AgentServerConfiguration) server
          .getFcInterface("agent-server-configuration");
    }
    catch (NoSuchInterfaceException e)
    {
      throw new IllegalArgumentException(
          "The givent server component has no \"agent-server-configuration\" interface");
    }
  }

  /**
   * Create and start an agent server
   * 
   * @param args the server configuration file and the name of the server to
   *          start
   * @throws Exception if error
   */
  public static void main(String[] args) throws Exception
  {
    if (args.length != 2)
    {
      usage();
    }
    createAgentServer(args[0], args[1]);
  }

  private static void usage()
  {
    System.err.println("Usage : AgentServer <a3server file> <server name>");
    System.err
        .println("  <a3server file> : URL of the a3server configuration file");
    System.err.println("  <server name>   : the name of the server to start");
    System.exit(1);
  }
}