/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

/*
 * Parts of this code are copyright:
 * 
 * The contents of this file are subject to the Joram Public License, as defined
 * by the file JORAM_LICENSE.TXT
 * 
 * You may not use this file except in compliance with the License. You may
 * obtain a copy of the License on the Objectweb web site (www.objectweb.org).
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific terms governing rights and limitations under the License.
 * 
 * The Original Code is Joram, including the java packages fr.dyade.aaa.agent,
 * fr.dyade.aaa.util, fr.dyade.aaa.ip, fr.dyade.aaa.mom, and fr.dyade.aaa.joram,
 * released May 24, 2000.
 * 
 * The Initial Developer of the Original Code is Dyade. The Original Code and
 * portions created by Dyade are Copyright Bull and Copyright INRIA. All Rights
 * Reserved.
 * 
 * The present code contributor is INRIA Rhone-Alpes.
 */

package org.objectweb.dream.aaa.agent;

import java.io.IOException;
import java.io.Serializable;

/**
 * <code>AgentIdStamp</code> class defines static members, variable and
 * functions, to manage the allocation of new identifiers. It locally maintains
 * a pair of counters, one for local agent server, one for remote agent server,
 * keeping track of the last allocated stamp for the given target domain. Stamps
 * are allocated in growing order and are never reused once allocated, even
 * after agents are deleted.
 * <p>
 * <hr>
 * Some initial stamps are reserved for statically identifying system and well
 * known services as factory. They are defined in <code>AgentId</code> class.
 */
class AgentIdStamp implements Serializable
{

  /** Static reference to local <code>AgentIdStamp</code> object. */
  protected static AgentIdStamp stamp = null;

  /** Stamp counter for local agent server. */
  private int                   local;
  /** Stamp counter for remote agent server. */
  private int                   remote;

  /* Initializes <code> AgentIdStamp </code> classe. */
  static
  {
    stamp = new AgentIdStamp();
  }

  AgentIdStamp()
  {
    local = AgentId.MAX_ID_STAMP;
    remote = AgentId.MAX_ID_STAMP;
  }

  /**
   * The <code>writeObject</code> method is responsible for writing the state
   * of the object for its particular class so that the corresponding
   * <code>readObject</code> method can restore it.
   * 
   * @param out the underlying output stream.
   */
  private void writeObject(java.io.ObjectOutputStream out) throws IOException
  {
    out.writeInt(local);
    out.writeInt(remote);
  }

  /**
   * The <code>readObject</code> is responsible for reading from the stream
   * and restoring the classes fields.
   * 
   * @param in the underlying input stream.
   */
  private void readObject(java.io.ObjectInputStream in) throws IOException,
      ClassNotFoundException
  {
    local = in.readInt();
    remote = in.readInt();
  }

  /**
   * Allocates a new stamp for the target agent server.
   * 
   * @param to The target agent server
   */
  int newStamp(short thisServerId, short to)
  {
    int current = (to == thisServerId) ? (++local) : (++remote);
    return current;
  }
}

/**
 * An <code>AgentId</code> allows for uniquely identifying and localizing an
 * agent throughout the distributed system. It defines variable members
 * describing the identifier and all needed function members to manipulate the
 * structure (creation, serialization, etc).
 * <p>
 * <hr>
 * However before describing the structure of an AgentId we must take into
 * account a second requirement: an agent may be created onto a remote agent
 * server, and the creating entity needs to know the new identifier created for
 * that agent. As agents live in an asynchronous world it is not so easy to get
 * back the identifier from the remote server. We decided instead to make the
 * creating entity responsible for creating the identifier.
 * <p>
 * The two requirements are then:
 * <ul>
 * <li>static localization of agents to allow the system to forward
 * notifications ;
 * <li>local generation of identifiers for remote agents.
 * </ul>
 * <hr>
 * The AgentId is then built of three parts:
 * <ul>
 * <li>the identification of the agent server hosting the creating agent (from
 * field),
 * <li>the identification of the agent server hosting the created agent (to
 * field),
 * <li>a stamp, local to the agent server hosting the creating agent (stamp
 * field) ; see <a href="AgentIdStamp.html">AgentIdStamp </a> class.
 * </ul>
 * The three fields form the unique global identifier of the agent, that is two
 * agents may share a common stamp, as long as their from or to fields differ.
 * The to field identifies the agent server hosting the agent, so it is used by
 * the channel to forward notifications to the agent.
 * <p>
 * 
 * @author Andr? Freyssinet
 * @see AgentIdStamp
 */
public final class AgentId implements Serializable, Cloneable
{

  // ***** ***** ***** *****
  // Reserved stamps for system services.
  // ***** ***** ***** *****

  /** Reserved stamp for NullId. */
  public static final int     NULL_ID_STAMP           = 0;
  /** Reserved stamp for factory <code>AgentId</code>. */
  public static final int     FACTORY_ID_STAMP        = 1;
  /** Reserved stamp for admin <code>AgentId</code>. */
  public static final int     ADMIN_ID_STAMP          = 2;
  /** Maximum reserved stamp for system services. */
  public static final int     MAX_SYSTEM_ID_STAMP     = 2;

  // ***** ***** ***** *****
  // Reserved stamps for well known services.
  // ***** ***** ***** *****

  /** Minimum reserved stamp for well known services. */
  public static final int     MIN_WKS_ID_STAMP        = MAX_SYSTEM_ID_STAMP + 1;
  /** Reserved stamp for name service <code>AgentId</code>. */
  public static final int     NAME_SERVICE_STAMP      = 4;
  /** Reserved stamp for scheduler service <code>AgentId</code>. */
  public static final int     SCHEDULER_SERVICE_STAMP = 5;
  /** Reserved stamp for fileTransfert service <code>AgentId</code>. */
  public static final int     FILE_TRANSFERT_STAMP    = 6;
  /** Reserved stamp for JNDI service <code>AgentId</code>. */
  public static final int     JNDI_SERVICE_STAMP      = 7;
  /** Maximum reserved stamp for well known services. */
  public static final int     MAX_WKS_ID_STAMP        = 1024;
  /** Maximum reserved stamp. */
  public static final int     MAX_ID_STAMP            = MAX_WKS_ID_STAMP;

  // ***** ***** ***** *****
  // Statically reserved id.
  // ***** ***** ***** *****

  /** null <code>AgentId</code>. */
  public static final AgentId NULL_ID                 = new AgentId((short) 0,
                                                          (short) 0,
                                                          NULL_ID_STAMP);

  //  Declares all fields transient in order to avoid useless
  // description of each during serialization.

  /** The identification of the agent server hosting the creating agent. */
  protected transient short   from;
  /** The identification of the agent server hosting the created agent. */
  protected transient short   to;
  /** The stamp, local to the agent server hosting the creating agent. */
  protected transient int     stamp;

  /**
   * The <code>writeObject</code> method is responsible for writing the state
   * of the object for its particular class so that the corresponding
   * <code>readObject</code> method can restore it.
   * 
   * @param out the underlying output stream.
   */
  private void writeObject(java.io.ObjectOutputStream out) throws IOException
  {
    out.writeShort(from);
    out.writeShort(to);
    out.writeInt(stamp);
  }

  /**
   * The <code>readObject</code> is responsible for reading from the stream
   * and restoring the classes fields.
   * 
   * @param in the underlying input stream.
   */
  private void readObject(java.io.ObjectInputStream in) throws IOException,
      ClassNotFoundException
  {
    from = in.readShort();
    to = in.readShort();
    stamp = in.readInt();
  }

  /**
   * Returns the <code>AgentId</code> for a remote factory agent.
   * 
   * @param sid remote server id.
   * @return the <code>AgentId</code> for a remote factory agent.
   */
  public static AgentId factoryId(short sid)
  {
    return new AgentId(sid, sid, FACTORY_ID_STAMP);
  }

  /**
   * Allocates an <code>AgentId</code> object for a new agent hosted by this
   * agent server.
   */
  AgentId(short local)
  {
    this(local, local);
  }

  /**
   * Allocates an <code>AgentId</code> object for a new agent hosted by
   * specified agent server.
   * 
   * @param to The identification of the agent server hosting the agent.
   */
  AgentId(short local, short to)
  {
    this(local, to, AgentIdStamp.stamp.newStamp(local, to));
  }

  /**
   * Allocates a new <code>AgentId</code> object.
   * 
   * @param from The identification of the agent server hosting the creating
   *          agent.
   * @param to The identification of the agent server hosting the agent.
   * @param stamp The stamp of agent.
   */
  public AgentId(short from, short to, int stamp)
  {
    this.from = from;
    this.to = to;
    this.stamp = stamp;
  }

  /**
   * returns the identification of the agent server hosting the creating agent.
   * 
   * @return the identification of the agent server hosting the creating agent.
   */
  public short getFrom()
  {
    return from;
  }

  /**
   * returns the identification of the agent server hosting the created agent.
   * 
   * @return the identification of the agent server hosting the created agent.
   */
  public short getTo()
  {
    return to;
  }

  /**
   * returns the stamp, local to the agent server hosting the creating agent.
   * 
   * @return the stamp, local to the agent server hosting the creating agent.
   */
  public int getStamp()
  {
    return stamp;
  }

  /**
   * Parses the string argument as an <code>AgentId</code>.
   * 
   * @param str the string
   * @return The <code>AgentId</code> object represented by the argument.
   */
  public static AgentId fromString(String str)
  {
    if (str == null)
    {
      return null;
    }
    if (str.charAt(0) != '#')
    {
      throw new IllegalArgumentException(str + ": bad id");
    }
    short from;
    short to;
    int stamp;
    try
    {
      String buf = str.substring(1);
      int index = buf.indexOf('.');
      from = Short.parseShort(buf.substring(0, index));
      buf = buf.substring(index + 1);
      index = buf.indexOf('.');
      to = Short.parseShort(buf.substring(0, index));
      buf = buf.substring(index + 1);
      stamp = Integer.parseInt(buf);
    }
    catch (Exception exc)
    {
      throw new IllegalArgumentException(str + ": " + exc);
    }
    return new AgentId(from, to, stamp);
  }

  /**
   * Returns a string representation of this <code>AgentId</code> object.
   * 
   * @return A string representation of this object.
   */
  public String toString()
  {
    return "#" + from + '.' + to + '.' + stamp;
  }

  /**
   * Returns a hashcode for this <code>AgentId</code> object.
   * 
   * @return a hash code value for this object, equal to the primitive
   *         <code>int</code> value represented by the stamp field.
   */
  public int hashCode()
  {
    return stamp;
  }

  /**
   * @return <code>true</code> if this id is equals to NullId;
   *         <code>false</code> otherwise.
   */
  public boolean isNullId()
  {
    return (stamp == NULL_ID_STAMP);
  }

  /**
   * Indicates whether some other agent id. is "equal to" this one. This method
   * returns <code>true</code> if and only if obj is an <code>AgentId</code>
   * and refer to the same agent (from, to and stamp fields are equals).
   * 
   * @param obj the reference object with which to compare.
   * @return <code>true</code> if this object is the same as the obj argument;
   *         <code>false</code> otherwise.
   */
  public boolean equals(Object obj)
  {
    return (obj instanceof AgentId) && (((AgentId) obj).from == from)
        && (((AgentId) obj).to == to) && (((AgentId) obj).stamp == stamp);
  }

  /**
   * Returns a clone of this agent id.
   * 
   * @return A clone of this agent id.
   */
  public Object clone()
  {
    try
    {
      return super.clone();
    }
    catch (CloneNotSupportedException e)
    {
      // this shouldn't happen, since we are Cloneable
      throw new InternalError();
    }
  }
}