/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.aaa.components.conf;

import java.util.ArrayList;
import java.util.List;

/**
 * Domain description
 */
public final class DomainDesc
{

  /** the name of this domain. */
  private String     name;

  /** the name of the message codec used in this domain. */
  private String     codecName;

  /** the name of the default ADL for network of this domain. */
  private String     defaultNetworkADL;

  /**
   * the server that must be used to access this domain. <code>null</code> if
   * this server instance is in this domain.
   */
  private ServerDesc gateway = null;

  /** The networks of this domain */
  private List       networks;

  /**
   * Creates a new domain with the specified name.
   * 
   * @param name the domain name
   * @param codecName the name of the codec.
   * @param defaultNetworkADL the network ADL name;
   */
  public DomainDesc(String name, String codecName, String defaultNetworkADL)
  {
    this.name = name;
    this.codecName = codecName;
    this.defaultNetworkADL = defaultNetworkADL;
    networks = new ArrayList();
  }

  /**
   * Adds a network in this domain
   * 
   * @param server the added network
   */
  public void addNetwork(NetworkDesc server)
  {
    networks.add(server);
  }

  /**
   * Returns the server that must be used to access this domain.
   * <code>null</code> if this server instance is in this domain.
   * 
   * @return the server that must be used to access this domain.
   *         <code>null</code> if this server instance is in this domain.
   */
  public ServerDesc getGateway()
  {
    return gateway;
  }

  /**
   * Returns the name of this domain
   * 
   * @return the name of this domain
   */
  public String getName()
  {
    return name;
  }

  /**
   * Returns the name of the message codec used in this domain.
   * 
   * @return the name of the message codec used in this domain.
   */
  public String getCodecName()
  {
    return codecName;
  }

  /**
   * Returns the name of the default ADL for network of this domain.
   * 
   * @return the name of the default ADL for network of this domain.
   */
  public String getDefaultNetworkADL()
  {
    return defaultNetworkADL;
  }

  /**
   * Sets the network that must be used to access this domain.
   * 
   * @param desc the network that must be used to access this domain.
   */
  public void setGateway(ServerDesc desc)
  {
    gateway = desc;
  }

  /**
   * Returns a list of networks in this domain
   * 
   * @return a list of networks in this domain
   */
  public List getNetworks()
  {
    return networks;
  }

}