/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.aaa.components.conf;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.List;

/**
 * Server description class
 */
public final class ServerDesc
{

  /** the name of this server */
  private String         name;

  /** the hostname of the host on which this server is started */
  private String         hostname;

  /** the inet address corresponding to the hostname */
  private InetAddress    inetAddress;

  /** the {@link NetworkDesc}used to access this server. */
  private NetworkDesc    accessNetwork;

  /** the list of {@link RepositoryDesc}deployed in this server */
  private List           repositories;

  /** the list of {@link NetworkDesc}in this server */
  private List           networks;

  /** a boolean to detect cycle domain. */
  private boolean        visited = false;

  /** the default repository for this server. */
  private RepositoryDesc defaultRepository;

  /**
   * Constructor
   * 
   * @param name the name of this server
   * @param hostname the hostname of the host on which this server is started
   * @throws UnknownHostException if the given hostname cannot be resolved
   */
  public ServerDesc(String name, String hostname) throws UnknownHostException
  {
    this.name = name;
    this.hostname = hostname;
    this.inetAddress = InetAddress.getByName(hostname);
    this.repositories = new ArrayList();
    this.networks = new ArrayList();
  }

  /**
   * Add a network for this server. This method only adds the network in the
   * list and will not modify or create any component.
   * 
   * @param network the network object to add
   */
  public void addNetwork(NetworkDesc network)
  {
    networks.add(network);
  }

  /**
   * Adds a repository in the list and evantualy sets as the default repository.
   * If no default repository is already defined, this one is setted as the
   * default repository, what ever the <code>defaultRepository</code>
   * parameter value. In other words, the first added repository is always
   * setted as the default repository, but can be overrided by a later added
   * repository.
   * 
   * @param repository a repository.
   * @param defaultRepository if <code>true</code>, this repository is setted
   *          as the default repository.
   */
  public void addRepository(RepositoryDesc repository, boolean defaultRepository)
  {
    this.repositories.add(repository);
    if (defaultRepository || this.defaultRepository == null)
    {
      this.defaultRepository = repository;
    }
  }

  /**
   * returns the hostname of the host on which this server is started
   * 
   * @return the hostname of the host on which this server is started
   */
  public String getHostname()
  {
    return hostname;
  }

  /**
   * returns the inet address corresponding to the hostname.
   * 
   * @return the inet address corresponding to the hostname.
   */
  public InetAddress getInetAddress()
  {
    return inetAddress;
  }

  /**
   * returns the name of this server
   * 
   * @return the name of this server
   */
  public String getName()
  {
    return name;
  }

  /**
   * returns the {@link NetworkDesc}used to access this server.
   * 
   * @return the {@link NetworkDesc}used to access this server.
   */
  public NetworkDesc getAccessNetwork()
  {
    return accessNetwork;
  }

  /**
   * Sets the {@link NetworkDesc}used to access this server.
   * 
   * @param network the {@link NetworkDesc}used to access this server.
   */
  public void setAccessNetwork(NetworkDesc network)
  {
    accessNetwork = network;
  }

  /**
   * returns the Repositories defined in this server.
   * 
   * @return the Repositories defined in this server.
   */
  public List getRepositories()
  {
    return repositories;
  }

  /**
   * returns the networks defined in the server.
   * 
   * @return the networks defined in the server.
   */
  public List getNetworks()
  {
    return networks;
  }

  /**
   * Returns the default repository for this server.
   * 
   * @return the default repository for this server.
   */
  public RepositoryDesc getDefaultRepository()
  {
    return defaultRepository;
  }

  /**
   * returns <code>true</code> if this server has already been visited.
   * 
   * @return <code>true</code> if this server has already been visited.
   */
  public boolean isVisited()
  {
    return visited;
  }

  /**
   * Set <code>visited</code> to true
   */
  public void visite()
  {
    visited = true;
  }

  /**
   * @see java.lang.Object#toString()
   */
  public String toString()
  {
    StringBuffer sb = new StringBuffer();
    sb.append("Server {name=").append(name).append(", ");
    sb.append("hostname=").append(hostname).append(", ");
    sb.append("network=").append(networks).append(", ");
    sb.append("Repositories={");
    Object[] tab = repositories.toArray();
    for (int i = 0; i < tab.length; i++)
    {
      if (tab[i] == defaultRepository)
      {
        sb.append("(default)");
      }
      sb.append(tab[i]);
      if (i != tab.length - 1)
      {
        sb.append(", ");
      }
    }
    sb.append("}}");
    return sb.toString();
  }
}