/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream;

import java.util.Hashtable;
import java.util.Map;

import org.objectweb.dream.control.logger.Loggable;
import org.objectweb.dream.control.logger.LoggerControllerRegister;
import org.objectweb.dream.util.NullLogger;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.julia.control.binding.ChainedIllegalBindingException;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * Base implementation of Dream components. This class is provided for
 * convenience. It is not mandatory for a component implementation to be a
 * subclasse of this class. <br>
 * This class provides implementation of <code>BindingController</code>
 * methods. Bindings are stored in a {@link java.util.Map}. When a component
 * has client interfaces, it is recommanded to override (at least) the
 * {@link BindingController#bindFc(String, Object) bindFc }method.
 */
public abstract class AbstractComponent
    implements
      LifeCycleController,
      BindingController,
      Loggable
{

  /** the {@link Component }controller interface reference of this component */
  public Component    weaveableC;

  /** the functional logger of this component */
  protected Logger    logger          = NullLogger.NULL_LOGGER;

  /**
   * the controller logger associated with the life cycle controller of this
   * component
   */
  protected Logger    lifeCycleLogger = NullLogger.NULL_LOGGER;

  /**
   * the controller logger associated with the binding controller of this
   * component
   */
  protected Logger    bindingLogger   = NullLogger.NULL_LOGGER;

  /** this hashtable contain all bindings of the component. */
  private Map         bindings;

  /**
   * this attribut give the state of Dream Component. <br>
   * the fcSstate value is {@link LifeCycleController#STARTED}
   * {@link LifeCycleController#STOPPED}
   */
  protected String    fcState;

  /** The {@link Component}description object associated with this component */
  protected Component componentDesc;

  /**
   * Constructor method create the hashtable of bindings and initialize fcState
   * to LifeCycleController.STOPPED.
   */
  public AbstractComponent()
  {
    bindings = new Hashtable();
    fcState = LifeCycleController.STOPPED;
  }

  /**
   * Initializes this component. This method is called during initialization of
   * the component controllers. <br>
   * This method can be overriden by sub classes with the following forme :
   * 
   * <pre>
   * 
   * protected void initComponent(Component componentItf)
   * {
   *   super.initComponent(componentItf);
   *   // initialisation code here ...
   * }
   * 
   * 
   * </pre>
   * 
   * @param componentItf the {@link Component }interface of this component.
   * @throws InitializationException if an error occurs.
   */
  protected void initComponent(Component componentItf)
      throws InitializationException
  {
    try
    {
      LoggerControllerRegister lcr = (LoggerControllerRegister) weaveableC
          .getFcInterface("/logger-controller-register");
      lcr.register(null, this);
      lcr.register("life-cycle", this);
      lcr.register("binding", this);
      logger.log(BasicLevel.DEBUG, "Component initialized.");
    }
    catch (NoSuchInterfaceException e)
    {
      // ignore
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the Loggable interface
  // ---------------------------------------------------------------------------

  /**
   * @see Loggable#setLogger(String, Logger)
   */
  public void setLogger(String name, Logger l)
  {
    if (name == null)
    {
      logger = l;
    }
    else if (name.equals("life-cycle"))
    {
      lifeCycleLogger = l;
    }
    else if (name.equals("binding"))
    {
      bindingLogger = l;
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the LifeCycleController interface
  // ---------------------------------------------------------------------------

  /**
   * @see LifeCycleController#getFcState()
   */
  public String getFcState()
  {
    if (lifeCycleLogger.isLoggable(BasicLevel.DEBUG))
    {
      lifeCycleLogger.log(BasicLevel.DEBUG, "getFcState: " + fcState);
    }
    return fcState;
  }

  /**
   * @see LifeCycleController#stopFc()
   */
  public void stopFc() throws IllegalLifeCycleException
  {
    fcState = LifeCycleController.STOPPED;
    lifeCycleLogger.log(BasicLevel.DEBUG, "stopped");
  }

  /**
   * @see LifeCycleController#startFc()
   */
  public void startFc() throws IllegalLifeCycleException
  {
    fcState = LifeCycleController.STARTED;
    lifeCycleLogger.log(BasicLevel.DEBUG, "started");
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see BindingController#lookupFc(String)
   */
  public Object lookupFc(String clientItfName) throws NoSuchInterfaceException
  {
    synchronized (bindings)
    {
      if (bindingLogger.isLoggable(BasicLevel.DEBUG))
      {
        bindingLogger.log(BasicLevel.DEBUG, "lookupFc on interface \""
            + clientItfName + "\"");
      }
      return bindings.get(clientItfName);
    }
  }

  /**
   * @see BindingController#bindFc(String, Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    synchronized (bindings)
    {
      if (bindingLogger.isLoggable(BasicLevel.DEBUG))
      {
        bindingLogger.log(BasicLevel.DEBUG, "bindFc on interface \""
            + clientItfName + "\"");
      }
      if (clientItfName.equals("component"))
      {
        weaveableC = (Component) serverItf;
        try
        {
          initComponent(weaveableC);
        }
        catch (InitializationException e)
        {
          logger.log(BasicLevel.ERROR,
              "An error occurs while initializing the component", e);
          throw new ChainedIllegalBindingException(e, weaveableC, null,
              "component", null, "An error occurs while initializing component");
        }
      }
      else
      {
        bindings.put(clientItfName, serverItf);
      }
    }
  }

  /**
   * @see BindingController#unbindFc(String)
   */
  public void unbindFc(String clientItfName) throws NoSuchInterfaceException,
      IllegalBindingException, IllegalLifeCycleException
  {
    synchronized (bindings)
    {
      if (bindingLogger.isLoggable(BasicLevel.DEBUG))
      {
        bindingLogger.log(BasicLevel.DEBUG, "unbindIf on interface \""
            + clientItfName + "\"");
      }
      bindings.remove(clientItfName);
    }
  }
}