/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.channel;

import java.io.IOException;
import java.net.BindException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.Push;
import org.objectweb.dream.message.codec.MessageCodec;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Abstract implementation of a TCP ChannelIn Component.
 */
public class AbstractTCPChannelInImpl extends AbstractComponent
    implements
      IPChannelInAttributeController
{

  /**
   * The default number of attempts to open a {@link ServerSocket }before
   * aborting.
   */
  private static final int SERVER_SOCKET_OPEN_RETRY = 20;

  /** The server interface to which this component is bound. */
  protected Push           outPushItf;
  /** The message encoder/decoder interface to which this component is bound. */
  protected MessageCodec   messageCodecItf;

  /** The listening port number; */
  protected int            portNum                  = -1;

  protected int            openRetry                = SERVER_SOCKET_OPEN_RETRY;

  // ---------------------------------------------------------------------------
  // Implementation of IPChannelInAttributeController interface
  // ---------------------------------------------------------------------------

  /**
   * @see IPChannelInAttributeController#getListeningPort()
   */
  public int getListeningPort()
  {
    return portNum;
  }

  /**
   * @see IPChannelInAttributeController#setListeningPort(int)
   */
  public void setListeningPort(int port) throws IllegalLifeCycleException
  {
    if (portNum != port && fcState.equals(STARTED))
    {
      throw new IllegalLifeCycleException(
          "cannot change listening port when the component is started");
    }
    portNum = port;
  }

  /**
   * @see IPChannelInAttributeController#getOpenRetry()
   */
  public int getOpenRetry()
  {
    return openRetry;
  }

  /**
   * @see IPChannelInAttributeController#setOpenRetry(int)
   */
  public void setOpenRetry(int retry)
  {
    openRetry = retry;
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{Push.OUT_PUSH_ITF_NAME, MessageCodec.ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(Push.OUT_PUSH_ITF_NAME))
    {
      outPushItf = (Push) serverItf;
    }
    else if (clientItfName.equals(MessageCodec.ITF_NAME))
    {
      messageCodecItf = (MessageCodec) serverItf;
    }
  }

  // ---------------------------------------------------------------------------
  // Utility methods
  // ---------------------------------------------------------------------------

  /**
   * Does not do anything.
   * 
   * @param socket
   * @throws IOException
   */
  protected void sendAck(Socket socket) throws IOException
  {
  }

  /**
   * This method creates and returns a server socket which uses all network
   * interfaces on the host, and is bound to the specified port. It may be
   * overloaded in subclass, in order to create particular subclasses of server
   * sockets.
   * 
   * @param port the port to listen to.
   * @return a server socket bound to the specified port.
   * @exception IOException for networking errors
   */
  protected ServerSocket createServerSocket(int port) throws IOException
  {
    for (int i = 0;; i++)
    {
      try
      {
        return new ServerSocket(port);
      }
      catch (BindException exc)
      {
        if (i > openRetry)
        {
          throw exc;
        }
        try
        {
          Thread.sleep(i * 250);
        }
        catch (InterruptedException ignored)
        {
          // ignored
        }
      }
    }
  }

  /**
   * Configures this socket using the socket options established for this
   * factory. It may be overloaded in subclass, in order to handle particular
   * subclasses of sockets
   * 
   * @param sock the socket.
   * @exception SocketException for networking errors
   */
  protected void setSocketOption(Socket sock) throws SocketException
  {
    // Don't use TCP data coalescing - ie Nagle's algorithm
    sock.setTcpNoDelay(true);
    // Read operation will block indefinitely until requested data arrives
    sock.setSoTimeout(0);
    // Set Linger-on-Close timeout.
    sock.setSoLinger(true, 60);
  }
}