/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): 
 */

package org.objectweb.dream.channel;

import java.io.IOException;
import java.io.ObjectOutputStream;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Map;

import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Basic implementation of a ChannelOut component, using TCP protocol. Messages
 * are sent to <i>one </i> destination, which can be changed using the
 * <code></code> controller.
 */
public class MonoDestinationTCPChannelOutImpl extends AbstractTCPChannelOutImpl
    implements
      MonoDestinationIPChannelOutAttributeController
{

  /** The inet address to which messages are sent. */
  protected InetAddress        inetAddress;

  /** The port to which messages are sent. */
  protected int                port;

  /** The socket that is used to send messages. */
  protected Socket             sock;

  /** The output stram that is used to send messages. */
  protected ObjectOutputStream oos;

  // ---------------------------------------------------------------------------
  // Implementation of the Push interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.Push#push(Message, Map)
   */
  public synchronized void push(Message message, Map context)
      throws PushException
  {

    try
    {
      if (logger.isLoggable(BasicLevel.DEBUG))
      {
        logger.log(BasicLevel.DEBUG, "try to send message -> " + message);
      }

      if (sock == null || !sock.isConnected())
      {
        if (oos != null)
        {
          try
          {
            oos.close();
          }
          catch (IOException ioe)
          {
            // ignore
          }
        }
        if (sock != null)
        {
          try
          {
            sock.close();
          }
          catch (IOException ioe)
          {
            // ignore
          }
        }
        sock = connectSocket(inetAddress, getPort());
        oos = new ObjectOutputStream(sock.getOutputStream());
      }

      logger.log(BasicLevel.DEBUG, "write message");

      messageCodecItf.encode(oos, message);
      receiveAck(sock);
      messageManagerItf.deleteMessage(message);
    }
    catch (IOException e)
    {
      throw new PushException(e);
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the MonoDestinationIPChannelOutAttributeController
  // interface
  // ---------------------------------------------------------------------------

  /**
   * @see MonoDestinationIPChannelOutAttributeController#getInetAddress()
   */
  public String getInetAddress()
  {
    return inetAddress.getCanonicalHostName();
  }

  /**
   * @see MonoDestinationIPChannelOutAttributeController#setInetAddress(String)
   */
  public void setInetAddress(String inetAddress) throws UnknownHostException,
      IOException
  {
    this.inetAddress = InetAddress.getByName(inetAddress);
    if (oos != null)
    {
      oos.close();
    }
    if (sock != null)
    {
      sock.close();
    }
  }

  /**
   * @see MonoDestinationIPChannelOutAttributeController#getPort()
   */
  public int getPort()
  {
    return port;
  }

  /**
   * @see MonoDestinationIPChannelOutAttributeController#setPort(int)
   */
  public void setPort(int port)
  {
    this.port = port;
  }

  // ---------------------------------------------------------------------------
  // Utility methods
  // ---------------------------------------------------------------------------

  /**
   * Does not do anything.
   * 
   * @param sock the socket to be used.
   * @throws IOException if an error occurs.
   */
  protected void receiveAck(Socket sock) throws IOException
  {
  }
}