/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.queue;

import java.util.LinkedList;
import java.util.Map;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.InterruptedPushException;
import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.manager.MessageManager;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Abstract implementation of a input push queue. It implements overflow
 * policies as defined in {@link PushQueueAttributeController}
 */
public abstract class AbstractPushQueueImpl extends AbstractComponent
    implements
      Push,
      PushQueueAttributeController
{

  protected MessageManager messageManagerItf;

  protected LinkedList     waitingList = new LinkedList();

  protected int            maxCapacity;

  protected String         overflowPolicy;

  /**
   * @see Push#push(Message, Map)
   */
  public synchronized void push(Message message, Map context)
      throws PushException
  {
    if (handleOverflow())
    {
      waitingList.add(message);
      this.notify();
    }
    else
    {
      messageManagerItf.deleteMessage(message);
    }
  }

  /**
   * Handle current overflow policy, and returns <code>true</code> if message
   * can be put in the waiting list.
   * 
   * @return <code>true</code> if message can be put in the waiting list.
   * @throws BufferOverflowException thrown when overflow policy is
   *           {@link PushQueueAttributeController#EXCEPTION_OVERFLOW_POLICY }
   *           and queue is full
   * @throws InterruptedPushException thrown when overflow policy is
   *           {@link PushQueueAttributeController#BLOCK_OVERFLOW_POLICY }and
   *           the queue is full and the thread has been interrupted.
   */
  protected boolean handleOverflow() throws BufferOverflowException,
      InterruptedPushException
  {
    if (waitingList.size() == maxCapacity)
    {
      if (overflowPolicy == BLOCK_OVERFLOW_POLICY)
      {
        while (waitingList.size() == maxCapacity)
        {
          try
          {
            this.wait();
          }
          catch (InterruptedException e)
          {
            throw new InterruptedPushException(e);
          }
        }
      }
      else if (overflowPolicy == DROP_OVERFLOW_POLICY)
      {
        logger.log(BasicLevel.INFO, "buffer overflow, drop new message");
        return false;
      }
      else if (overflowPolicy == EXCEPTION_OVERFLOW_POLICY)
      {
        throw new BufferOverflowException("Overflow in buffer");
      }
    }
    return true;
  }

  // ---------------------------------------------------------------------------
  // Implementation of PushQueueAttributeController interface
  // ---------------------------------------------------------------------------

  /**
   * @see PushQueueAttributeController#getOverflowPolicy()
   */
  public String getOverflowPolicy()
  {
    return overflowPolicy;
  }

  /**
   * @see PushQueueAttributeController#setOverflowPolicy(String)
   */
  public synchronized void setOverflowPolicy(String policy)
  {
    // set overflow policy attribute reference to interface constant to test
    // policy efficiently.
    if (policy.equals(BLOCK_OVERFLOW_POLICY))
    {
      overflowPolicy = BLOCK_OVERFLOW_POLICY;
    }
    else if (policy.equals(DROP_OVERFLOW_POLICY))
    {
      overflowPolicy = DROP_OVERFLOW_POLICY;
    }
    else if (policy.equals(EXCEPTION_OVERFLOW_POLICY))
    {
      overflowPolicy = EXCEPTION_OVERFLOW_POLICY;
    }
    else
    {
      throw new IllegalArgumentException("Unknown overflow policy : " + policy);
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of QueueAttributeController interface
  // ---------------------------------------------------------------------------

  /**
   * @see QueueAttributeController#getCurrentSize()
   */
  public synchronized int getCurrentSize()
  {
    return waitingList.size();
  }

  /**
   * @see QueueAttributeController#getMaxCapacity()
   */
  public synchronized int getMaxCapacity()
  {
    return maxCapacity;
  }

  /**
   * @see QueueAttributeController#setMaxCapacity(int)
   */
  public synchronized void setMaxCapacity(int maxCapacity)
  {
    int previousCapacity = this.maxCapacity;
    this.maxCapacity = maxCapacity;
    if (overflowPolicy == BLOCK_OVERFLOW_POLICY
        && maxCapacity > previousCapacity)
    {
      // notify waiting push.
      this.notifyAll();
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
  }

}